from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sspo_db.model.activity.models import *
from sspo_db.service.base_service import BaseService

class ScrumDevelopmentTaskService(BaseService):
    def __init__(self):
        super(ScrumDevelopmentTaskService,self).__init__(ScrumDevelopmentTask)

class ScrumIntentedDevelopmentTaskService(BaseService):
    def __init__(self):
        super(ScrumIntentedDevelopmentTaskService,self).__init__(ScrumIntentedDevelopmentTask)

class ScrumPerformedDevelopmentTaskService(BaseService):
    def __init__(self):
        super(ScrumPerformedDevelopmentTaskService,self).__init__(ScrumPerformedDevelopmentTask)

class DevelopmentTaskTypeService(BaseService):
    def __init__(self):
        super(DevelopmentTaskTypeService,self).__init__(DevelopmentTaskType)
    
    def retrive_by_name (self, name):
        return self.session.query(DevelopmentTaskType).filter(DevelopmentTaskType.name.like (name)).first()

class PriorityService(BaseService):
    
    def __init__(self):
        super(PriorityService,self).__init__(Priority)
    
    def retrive_by_name (self, name):
        return self.session.query(Priority).filter(Priority.name.like (name)).first()

class RiskService(BaseService):

    def __init__(self):
        super(RiskService,self).__init__(Risk)
    
    def retrive_by_name (self, name):
        return self.session.query(Risk).filter(Risk.name.like (name)).first()







from sspo_db.config.config import Base
from sqlalchemy import Column ,ForeignKey, Integer, Table
from sqlalchemy.orm import relationship

association_sprint_scrum_development_table = Table('sprint_scrum_development_task', Base.metadata,
    Column('scrum_development_task_id', Integer, ForeignKey('scrum_development_task.id')),
    Column('sprint_id', Integer, ForeignKey('sprint.id'))
)

association_sprint_backlog_scrum_development_activity_table = Table('sprint_backlog_scrum_development_task', Base.metadata,
    Column('scrum_development_task_id', Integer, ForeignKey('scrum_development_task.id')),
    Column('sprint_backlog_id', Integer, ForeignKey('sprint_backlog.id'))
)

association_development_task_team_member_table = Table('team_member_scrum_development_task', Base.metadata,
    Column('scrum_development_task_id', Integer, ForeignKey('scrum_development_task.id')),
    Column('team_member_id', Integer, ForeignKey('team_member.id'))
)

association_atomic_user_story_sprint_backlog_table = Table('atomic_user_story_sprint_backlog', Base.metadata,
    Column('atomic_user_story_id', Integer, ForeignKey('atomic_user_story.id')),
    Column('sprint_backlog_id', Integer, ForeignKey('sprint_backlog.id'))
)

association_user_story_sprint_team_member_table = Table('user_story_team_member', Base.metadata,
    Column('user_story_id', Integer, ForeignKey('user_story.id')),
    Column('team_member_id', Integer, ForeignKey('team_member.id'))
)


from sspo_db.config.base import Entity
from sqlalchemy import Column, Boolean ,ForeignKey, Integer, DateTime, String
from sqlalchemy.orm import relationship
from sqlalchemy_utils import EmailType
from sspo_db.model.stakeholder.models import Person

class Organization(Entity):
    is_instance_of = "eo.organization" 
    __tablename__ = "organization"

    scrum_project = relationship("ScrumProject", back_populates="organization")
    team = relationship("Team", back_populates="organization")
    people = relationship(Person, back_populates="organization")
    configuration = relationship("Configuration", back_populates="organization") 

class Team(Entity):
    is_instance_of = "eo.team"
    __tablename__ = "team"

    type = Column(String(50))
    
    organization_id = Column(Integer, ForeignKey('organization.id'))
    organization = relationship("Organization", back_populates="team")

    team_members = relationship("TeamMember", back_populates="team")

    __mapper_args__ = {
        'polymorphic_identity':'team',
        'polymorphic_on':type
    }

class ScrumTeam(Team):
    is_instance_of = "eo.team.complex"
    #is_instance_of = "spo.stakeholder.project_team"
    __tablename__ = "scrum_team"
    
    id = Column(Integer, ForeignKey('team.id'), primary_key=True)

    scrum_project = Column(Integer, ForeignKey('scrum_project.id'))
    #development_team = relationship("DevelopmentTeam", back_populates="scrum_team")
    
    __mapper_args__ = {
        'polymorphic_identity':'scrum_team',
    }

class DevelopmentTeam(Team):
    is_instance_of = "eo.team.atomic"
    #is_instance_of = "spo.stakeholder.project_team"
    __tablename__ = "development_team"

    id = Column(Integer, ForeignKey('team.id'), primary_key=True)
    scrum_team_id = Column(Integer, ForeignKey('scrum_team.id'))
    #scrum_team = relationship("ScrumTeam", back_populates="development_team", foreign_keys=[scrum_team_id])
    
    
    __mapper_args__ = {
        'polymorphic_identity':'development_team',
    }



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sspo_db.config.config import session
from sspo_db.model.core.models import ApplicationReference

class BaseService():
    
    def __init__(self, object):
        self.session = session
        self.object = object
        self.type = self.object.__tablename__
    
    def get_all(self):
        return self.session.query(self.object).order_by(self.object.id)
    
    def get_by_uuid(self, uuid):
        return self.session.query(self.object).filter(self.object.uuid == uuid).first()
        
    def create(self, object):
        self.session.add(object)
        self.session.commit()

    def update (self, object):
        
        self.session.query(self.object).filter(self.object.id == object.id).update({column: getattr(object, column) for column in self.object.__table__.columns.keys()})
        self.session.commit()

    def delete(self, object):
        self.session.delete(object)
        self.session.commit()



from sspo_db.model.stakeholder.models import Person, Developer, ScrumMaster
from sspo_db.service.stakeholder.service import PersonService, DeveloperService, ScrumMasterService, TeamMemberService
from sspo_db.application.core.application import ApplicationApplicationReference
from sspo_db.application.abstract_application import AbstractApplication
from pprint import pprint

class ApplicationPerson(AbstractApplication):
    
    def __init__(self):
        super().__init__(PersonService())
    
    def get_all(self, organization_uuid):
        return self.service.get_all(organization_uuid)
        
class ApplicationTeamMember(AbstractApplication):
    
    def __init__(self):
        super().__init__(TeamMemberService())
        
    def retrive_by_external_id_and_project_name(self, external_id, project_name):
        application_person = ApplicationPerson()
        person = application_person.retrive_by_external_uuid(external_id)
        if person is None:
            return None
        return self.service.retrive_by_external_id_and_project_name(person,project_name)

class ApplicationDeveloper(AbstractApplication):

    def __init__(self):
        super().__init__(DeveloperService())
    
    def create_with_project_name(self, external_id, project_name):
        application_person = ApplicationPerson()
        person = application_person.retrive_by_external_uuid(external_id)
        if person is not None:
            return self.service.create_with_project_name(person, project_name)
        return None

        
        
class ApplicationScrumMaster(AbstractApplication):
    
    def __init__(self):
        super().__init__(ScrumMasterService())
        
    


from sspo_db.application.abstract_application import AbstractApplication
from sspo_db.service.stakeholder_participation.service import *

class ApplicationIntentedStakeholderParticipation(AbstractApplication):
    
    def __init__(self):
        super().__init__(PerformedFragmentParticipation())
    
class ApplicationPerformedStakeholderParticipation(AbstractApplication):
    
    def __init__(self):
        super().__init__(PerformedStakeholderParticipationService())

class ApplicationPerformedFragmentParticipation(AbstractApplication):
    
    def __init__(self):
        super().__init__(PerformedFragmentParticipationService())




from sspo_db.application.abstract_application import AbstractApplication
from sspo_db.service.organization.service import OrganizationService, TeamService, DevelopmentTeamService, ScrumTeamService

class ApplicationOrganization(AbstractApplication):

    def __init__(self):
        super().__init__(OrganizationService())

class ApplicationTeam(AbstractApplication):

    def __init__(self):
        super().__init__(TeamService())

class ApplicationDevelopmentTeam(AbstractApplication):

    def __init__(self):
        super().__init__(DevelopmentTeamService())

class ApplicationScrumTeam(AbstractApplication):

    def __init__(self):
        super().__init__(ScrumTeamService())


from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sspo_db.model.process.models import *
from sspo_db.model.organization.models import *
from sspo_db.service.base_service import BaseService
from sspo_db.service.core.service import ApplicationReferenceService

class ScrumProjectService(BaseService):
    def __init__(self):
        super(ScrumProjectService,self).__init__(ScrumProject)
    
    
class ScrumComplexProjectService(BaseService):
    def __init__(self):
        super(ScrumComplexProjectService,self).__init__(ScrumComplexProject)
    
    def get_all(self, organization_uuid):
        return self.session.query(self.object).join(Organization).filter(Organization.uuid == organization_uuid).order_by(self.object.id)
        
class ScrumAtomicProjectService(BaseService):
    def __init__(self):
        super(ScrumAtomicProjectService,self).__init__(ScrumAtomicProject)
    
    def get_all(self, organization_uuid):
        return self.session.query(self.object).join(Organization).filter(Organization.uuid == organization_uuid).order_by(self.object.id)

class ScrumProcessService(BaseService):
    def __init__(self):
        super(ScrumProcessService,self).__init__(ScrumProcess)

class ProductBacklogDefinitionService(BaseService):
    def __init__(self):
        super(ProductBacklogDefinitionService,self).__init__(ProductBacklogDefinition)

class SprintService(BaseService):
    def __init__(self):
        super(SprintService,self).__init__(Sprint)

    def retrive_by_name_and_project_name(self, sprint_name, project_name):
        return self.session.query(Sprint).join(ScrumProcess).join(ScrumProject).filter(ScrumProject.name.like(project_name), Sprint.name.like(sprint_name)).first()
    
    def retrive_limbo(self,project_uuid):
        return self.session.query(Sprint).join(ScrumProcess).join(ScrumProject).filter(ScrumProject.uuid == project_uuid, Sprint.name.like("limbo")).first()

class CerimonyService(BaseService):
    def __init__(self):
        super(CerimonyService,self).__init__(Cerimony)

class PlanningMeetingService(BaseService):
    def __init__(self):
        super(PlanningMeetingService,self).__init__(PlanningMeeting)

class DailyStandupMeetingService(BaseService):
    def __init__(self):
        super(DailyStandupMeetingService,self).__init__(DailyStandupMeeting)

class ReviewMeetingService(BaseService):
    def __init__(self):
        super(ReviewMeetingService,self).__init__(ReviewMeeting)

class RetrospectiveMeetingService(BaseService):
    def __init__(self):
        super(RetrospectiveMeetingService,self).__init__(RetrospectiveMeeting)


from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sspo_db.model.stakeholder_participation.models import *
from sspo_db.service.base_service import BaseService

class IntentedStakeholderParticipationService(BaseService):
    def __init__(self):
        super(IntentedStakeholderParticipationService,self).__init__(IntentedStakeholderParticipation)

class PerformedStakeholderParticipationService(BaseService):
    def __init__(self):
        super(PerformedStakeholderParticipationService,self).__init__(PerformedStakeholderParticipation)

class PerformedFragmentParticipationService(BaseService):
    def __init__(self):
        super(PerformedFragmentParticipationService,self).__init__(PerformedFragmentParticipation)



from sspo_db.config.base import Entity
from sspo_db.config.config import Base
from sqlalchemy import Column, Boolean ,ForeignKey, Float, Integer, DateTime, String, Table
from sqlalchemy.orm import relationship
from sspo_db.model.relationship.model import association_user_story_sprint_team_member_table, association_sprint_backlog_scrum_development_activity_table,  association_atomic_user_story_sprint_backlog_table

class ProductBacklog(Entity):
    is_instance_of = "spo.document"
    __tablename__ = "product_backlog"

    product_backlog_definition = Column(Integer, ForeignKey('product_backlog_definition.id')) 


class UserStory(Entity):
    is_instance_of = "rsro.requirements.artifact"
    __tablename__ = "user_story"

    product_backlog =  Column(Integer, ForeignKey('product_backlog.id'))
    
    created_by = Column(Integer, ForeignKey('team_member.id')) 
    activated_by = Column(Integer, ForeignKey('team_member.id'))
    closed_by = Column(Integer, ForeignKey('team_member.id'))
    assigned_by = relationship("TeamMember", secondary=association_user_story_sprint_team_member_table)
    
    story_points = Column(Float(),nullable=True) 

    created_date = Column(DateTime)
    activated_date = Column(DateTime)
    closed_date = Column(DateTime)
    resolved_date = Column(DateTime)
    
    type = Column(String(50))

    __mapper_args__ = {
        'polymorphic_identity':'user_story',
        'polymorphic_on':type
    }
    
class Epic(UserStory):

    __tablename__ = "epic"
    id = Column(Integer, ForeignKey('user_story.id'), primary_key=True)

    epic = Column(Integer, ForeignKey('epic.id'))

    __mapper_args__ = {
        'polymorphic_identity':'epic',
    }

class AtomicUserStory(UserStory):

    __tablename__ = "atomic_user_story"
    
    id = Column(Integer, ForeignKey('user_story.id'), primary_key=True)
    epic = Column(Integer, ForeignKey('epic.id'))
    sprint_backlogs = relationship("SprintBacklog", secondary=association_atomic_user_story_sprint_backlog_table, back_populates="atomic_user_stories")

    __mapper_args__ = {
        'polymorphic_identity':'atomic_user_story',
    }

class SprintBacklog(Entity):
    is_instance_of = "spo.document"
    __tablename__ = "sprint_backlog"

    sprint = Column(Integer, ForeignKey('sprint.id'))
    planning_meeting = Column(Integer, ForeignKey('planning_meeting.id'))
    
    scrum_development_tasks = relationship("ScrumDevelopmentTask", secondary=association_sprint_backlog_scrum_development_activity_table, back_populates="sprint_backlogs")
    atomic_user_stories = relationship("AtomicUserStory", secondary=association_atomic_user_story_sprint_backlog_table, back_populates="sprint_backlogs")


class AcceptanceCriterion(Entity):
    is_instance_of = "rsro.requirements"
    __tablename__ = "acceptance_criterion"

    user_story =  Column(Integer, ForeignKey('user_story.id'))

    type = Column(String(50))

    __mapper_args__ = {
        'polymorphic_identity':'acceptance_criterion',
        'polymorphic_on':type
    }
    

class NonFunctionalAcceptanceCriterion(AcceptanceCriterion):
    is_instance_of = "rsro.non_functional_requirements"
    __tablename__ = "non_functional_acceptance_criterion"
    
    id = Column(Integer, ForeignKey('acceptance_criterion.id'), primary_key=True)

    __mapper_args__ = {
        'polymorphic_identity':'non_functional_acceptance_criterion',
    }

class FunctionalAcceptanceCriterion(AcceptanceCriterion):
    is_instance_of = "rsro.functional_requirements"
    __tablename__ = "functional_acceptance_criterion"
    
    id = Column(Integer, ForeignKey('acceptance_criterion.id'), primary_key=True)

    __mapper_args__ = {
        'polymorphic_identity':'functional_acceptance_criterion',
    }



from sspo_db.application.abstract_application import AbstractApplication
from sspo_db.service.artifact.service import UserStoryService, EpicService, AtomicUserStoryService, ProductBacklogService, SprintBacklogService


class ApplicationEpic(AbstractApplication):
    
    def __init__(self):
        super().__init__(EpicService())

class ApplicationUserStory(AbstractApplication):
    
    def __init__(self):
        super().__init__(UserStoryService())

class ApplicationAtomicUserStory(AbstractApplication):
    
    def __init__(self):
        super().__init__(AtomicUserStoryService())

class ApplicationProductBacklog(AbstractApplication):

    def __init__ (self):
        super().__init__(ProductBacklogService())
    
    def retrive_by_project_name(self, project_name):
        return self.service.retrive_by_project_name(project_name)

class ApplicationSprintBacklog(AbstractApplication):

    def __init__ (self):
        super().__init__(SprintBacklogService())
        
    def retrive_by_name_and_project_name(self, sprint_name, project_name):
        return self.service.retrive_by_name_and_project_name( sprint_name, project_name)
    


from sspo_db.config.base import Entity
from sspo_db.config.config import Base
from sqlalchemy import Column, Boolean ,ForeignKey, Integer, DateTime, String, Table, Float
from sqlalchemy.orm import relationship
from sspo_db.model.process.models import Sprint
from sspo_db.model.stakeholder.models import TeamMember
from sspo_db.model.artifact.models import SprintBacklog
from sspo_db.model.relationship.model import association_sprint_backlog_scrum_development_activity_table,  association_sprint_scrum_development_table, association_development_task_team_member_table


class ScrumDevelopmentTask(Entity):
    
    is_instance_of = "spo.activity"
    __tablename__ = "scrum_development_task"

    created_date = Column(DateTime)
    created_by = Column(Integer, ForeignKey('team_member.id'))
    assigned_by = relationship("TeamMember", secondary=association_development_task_team_member_table)

    story_points = Column(Float(),nullable=True) 
    type = Column(String(50))

    sprints = relationship(Sprint, 
                            secondary=association_sprint_scrum_development_table, 
                            back_populates="scrum_development_tasks")

    sprint_backlogs = relationship(SprintBacklog, 
                        secondary=association_sprint_backlog_scrum_development_activity_table, 
                        back_populates="scrum_development_tasks")
    
    atomic_user_story = Column(Integer, ForeignKey('atomic_user_story.id'))
    
    __mapper_args__ = {
        'polymorphic_identity':'scrum_development_task',
        'polymorphic_on':type
    }
    
class DevelopmentTaskType(Entity):

    __tablename__ = "development_task_type"

    Analysis = "analysis"
    deployment = "deployment"
    design = "design"
    development = "development"
    documentation = "documentation"
    requirements = "requirements"
    testing = "testing"
    
class Priority(Entity):
    __tablename__ = "priority"
    normal = "normal"
    medium = "medium"
    high = "high"
    
class Risk(Entity):
    __tablename__ = "risk"

    high = "high"
    medium = "medium"
    normal = "normal"

class ScrumIntentedDevelopmentTask(ScrumDevelopmentTask):

    is_instance_of = "spo.intended.activity.x"
    __tablename__ = "scrum_intented_development_task"

    id = Column(Integer, ForeignKey('scrum_development_task.id'), primary_key=True)
    
    type_activity = Column(Integer, ForeignKey('development_task_type.id'))
    priority = Column(Integer, ForeignKey('priority.id'))
    risk = Column(Integer, ForeignKey('risk.id'))

    __mapper_args__ = {
        'polymorphic_identity':'scrum_intented_development_task',
    }
    
class ScrumPerformedDevelopmentTask(ScrumDevelopmentTask):
    is_instance_of = "spo.performed.activity.x"
    __tablename__ = "scrum_performed_development_task"

    id = Column(Integer, ForeignKey('scrum_development_task.id'), primary_key=True)
    
    closed_date = Column(DateTime)
    activated_date = Column(DateTime)
    resolved_date = Column(DateTime)

    activated_by = Column(Integer, ForeignKey('team_member.id'))
    closed_by = Column(Integer, ForeignKey('team_member.id'))
    resolved_by = Column(Integer, ForeignKey('team_member.id'))
    
    caused_by = Column(Integer, ForeignKey('scrum_intented_development_task.id'))

    __mapper_args__ = {
        'polymorphic_identity':'scrum_performed_development_task',
    }



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sspo_db.model.stakeholder.models import *
from sspo_db.model.organization.models import ScrumTeam, DevelopmentTeam
from sspo_db.model.process.models import ScrumProject
from sspo_db.model.organization.models import Organization
from sspo_db.service.core.service import ApplicationReferenceService
from sspo_db.service.base_service import BaseService

class PersonService(BaseService):

    def __init__(self):
        super(PersonService,self).__init__(Person)
    
    def get_all(self, organization_uuid):
        return self.session.query(self.object).join(Organization).filter(Organization.uuid == organization_uuid).order_by(self.object.id)

class TeamMemberService(BaseService):
    def __init__(self):
        super(TeamMemberService,self).__init__(TeamMember)
        
    def retrive_by_external_id_and_project_name (self, person, project_name):
        
        scrum_team = self.session.query(ScrumTeam).join(ScrumProject).filter(ScrumProject.name.like (project_name)).first()
        team_member = self.session.query(TeamMember).filter(TeamMember.person == person, TeamMember.team == scrum_team).first()
        
        if team_member is None and scrum_team is not None: 
            developmen_team = self.session.query(DevelopmentTeam).filter(DevelopmentTeam.scrum_team_id == scrum_team.id).first()
            team_member = self.session.query(TeamMember).join(DevelopmentTeam).filter(TeamMember.team_id == developmen_team.id, TeamMember.person == person, DevelopmentTeam.scrum_team_id == scrum_team.id).first()
        
        return team_member

class DeveloperService(BaseService):
    def __init__(self):
        super(DeveloperService,self).__init__(Developer)
    
    def create_with_project_name(self, person, project_name):
        
        scrum_team = self.session.query(ScrumTeam).join(ScrumProject).filter(ScrumProject.name.like (project_name)).first()
        developmen_team = self.session.query(DevelopmentTeam).filter(DevelopmentTeam.scrum_team_id == scrum_team.id).first()

        developer = Developer()
        developer.name = person.name
        developer.description = person.description
        developer.person = person
        developer.team_role = ""
        developer.team = developmen_team
        return developer

    
class ScrumMasterService(BaseService):
    def __init__(self):
        super(ScrumMasterService,self).__init__(ScrumMaster)

class ProductOwnerService(BaseService):
    def __init__(self):
        super(ProductOwnerService,self).__init__(ProductOwner)

class ClientService(BaseService):
    def __init__(self):
        super(ClientService,self).__init__(Client)


from sspo_db.application.activity.application import *
from sspo_db.application.artifact.application import *
from sspo_db.application.core.application import *
from sspo_db.application.organization.application import *
from sspo_db.application.process.application import *
from sspo_db.application.stakeholder.application import *
from sspo_db.application.stakeholder_participation.application import *
import factory

#stakeholder participation
class IntentedStakeholderParticipationFactory(factory.Factory):
    class Meta:
        model = ApplicationIntentedStakeholderParticipation

class PerformedStakeholderParticipationFactory(factory.Factory):
    class Meta:
        model = ApplicationPerformedStakeholderParticipation

class PerformedFragmentParticipationFactory(factory.Factory):
    class Meta:
        model = ApplicationPerformedFragmentParticipation

# stakeholders
class PersonFactory(factory.Factory):
    class Meta:
        model = ApplicationPerson

class TeamMemberFactory(factory.Factory):
    class Meta:
        model = ApplicationTeamMember

class DeveloperFactory(factory.Factory):
    class Meta:
        model = ApplicationDeveloper

class ScrumMasterFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumMaster

# Process
class ScrumProjectFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumProject

class ScrumComplexProjectFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumComplexProject

class ScrumAtomicProjectFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumAtomicProject

class ScrumProcessFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumProcess

class ProductBacklogDefinitionFactory(factory.Factory):
    class Meta:
        model = ApplicationProductBacklogDefinition

class SprintFactory(factory.Factory):
    class Meta:
        model = ApplicationSprint

# Organization
class OrganizationFactory(factory.Factory):
    class Meta:
        model = ApplicationOrganization

class TeamFactory(factory.Factory):
    class Meta:
        model = ApplicationTeam

class ScrumTeamFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumTeam

class DevelopmentTeamFactory(factory.Factory):
    class Meta:
        model = ApplicationDevelopmentTeam


## Core
class ApplicationTypeFactory(factory.Factory):
    class Meta:
        model = ApplicationApplicationType

class ApplicationFactory(factory.Factory):
    class Meta:
        model = ApplicationApplication

class ConfigurationFactory(factory.Factory):
    class Meta:
        model = ApplicationConfiguration

class ApplicationReferenceFactory(factory.Factory):
    class Meta:
        model = ApplicationApplicationReference


## Artifact

class ProductBacklogFactory(factory.Factory):
    class Meta:
        model = ApplicationProductBacklog

class UserStoryFactory(factory.Factory):
    class Meta:
        model = ApplicationUserStory

class EpicFactory(factory.Factory):
    class Meta:
        model = ApplicationEpic

class AtomicUserStoryFactory(factory.Factory):
    class Meta:
        model = ApplicationAtomicUserStory

class SprintBacklogFactory(factory.Factory):
    class Meta:
        model = ApplicationSprintBacklog

### Activiy ####
class ScrumDevelopmentTaskFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumDevelopmentTask

class DevelopmentTaskTypeFactory(factory.Factory):
    class Meta:
        model = ApplicationDevelopmentTaskType

class PriorityFactory(factory.Factory):
    class Meta:
        model = ApplicationPriority

class RiskFactory(factory.Factory):
    class Meta:
        model = ApplicationRisk

class ScrumIntentedDevelopmentTaskFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumIntentedDevelopmentTask

class ScrumPerformedDevelopmentTaskFactory(factory.Factory):
    class Meta:
        model = ApplicationScrumPerformedDevelopmentTask


from sspo_db.model.activity.models import *
from sspo_db.model.artifact.models import *
from sspo_db.model.core.models import *
from sspo_db.model.organization.models import *
from sspo_db.model.process.models import *
from sspo_db.model.stakeholder.models import *
from sspo_db.model.stakeholder_participation.models import *
import factory

# stakeholders_participation
class IntentedStakeholderParticipationFactory(factory.Factory):
    class Meta:
        model = IntentedStakeholderParticipation

class PerformedStakeholderParticipationFactory(factory.Factory):
    class Meta:
        model = PerformedStakeholderParticipation

class PerformedFragmentParticipationFactory(factory.Factory):
    class Meta:
        model = PerformedFragmentParticipation


# stakeholders
class PersonFactory(factory.Factory):
    class Meta:
        model = Person

class TeamMemberFactory(factory.Factory):
    class Meta:
        model = TeamMember

class DeveloperFactory(factory.Factory):
    class Meta:
        model = Developer

class ScrumMasterFactory(factory.Factory):
    class Meta:
        model = ScrumMaster

class ProductOwnerFactory(factory.Factory):
    class Meta:
        model = ProductOwner

class ClientFactory(factory.Factory):
    class Meta:
        model = Client

# Process
class ScrumProjectFactory(factory.Factory):
    class Meta:
        model = ScrumProject

class ScrumComplexProjectFactory(factory.Factory):
    class Meta:
        model = ScrumComplexProject

class ScrumAtomicProjectFactory(factory.Factory):
    class Meta:
        model = ScrumAtomicProject

class ScrumProcessFactory(factory.Factory):
    class Meta:
        model = ScrumProcess

class ProductBacklogDefinitionFactory(factory.Factory):
    class Meta:
        model = ProductBacklogDefinition

class SprintFactory(factory.Factory):
    class Meta:
        model = Sprint

class CerimonyFactory(factory.Factory):
    class Meta:
        model = Cerimony

class PlanningMeetingFactory(factory.Factory):
    class Meta:
        model = PlanningMeeting

class DailyStandupMeetingFactory(factory.Factory):
    class Meta:
        model = DailyStandupMeeting

class ReviewMeetingFactory(factory.Factory):
    class Meta:
        model = ReviewMeeting

class RetrospectiveMeetingFactory(factory.Factory):
    class Meta:
        model = RetrospectiveMeeting

# Organization
class OrganizationFactory(factory.Factory):
    class Meta:
        model = Organization

class TeamFactory(factory.Factory):
    class Meta:
        model = Team

class ScrumTeamFactory(factory.Factory):
    class Meta:
        model = ScrumTeam

class DevelopmentTeamFactory(factory.Factory):
    class Meta:
        model = DevelopmentTeam


## Core
class ApplicationTypeFactory(factory.Factory):
    class Meta:
        model = ApplicationType

class ApplicationFactory(factory.Factory):
    class Meta:
        model = Application

class ConfigurationFactory(factory.Factory):
    class Meta:
        model = Configuration

class ApplicationReferenceFactory(factory.Factory):
    class Meta:
        model = ApplicationReference


## Artifact

class ProductBacklogFactory(factory.Factory):
    class Meta:
        model = ProductBacklog

class UserStoryFactory(factory.Factory):
    class Meta:
        model = UserStory

class EpicFactory(factory.Factory):
    class Meta:
        model = Epic

class AtomicUserStoryFactory(factory.Factory):
    class Meta:
        model = AtomicUserStory

class SprintBacklogFactory(factory.Factory):
    class Meta:
        model = SprintBacklog

class AcceptanceCriterionFactory(factory.Factory):
    class Meta:
        model = AcceptanceCriterion

class NonFunctionalAcceptanceCriterionFactory(factory.Factory):
    class Meta:
        model = NonFunctionalAcceptanceCriterion

class FunctionalAcceptanceCriterionFactory(factory.Factory):
    class Meta:
        model = FunctionalAcceptanceCriterion

### Activiy ####
class ScrumDevelopmentTaskFactory(factory.Factory):
    class Meta:
        model = ScrumDevelopmentTask

class DevelopmentTaskTypeFactory(factory.Factory):
    class Meta:
        model = DevelopmentTaskType

class PriorityFactory(factory.Factory):
    class Meta:
        model = Priority

class RiskFactory(factory.Factory):
    class Meta:
        model = Risk

class ScrumIntentedDevelopmentTaskFactory(factory.Factory):
    class Meta:
        model = ScrumIntentedDevelopmentTask

class ScrumPerformedDevelopmentTaskFactory(factory.Factory):
    class Meta:
        model = ScrumPerformedDevelopmentTask


from abc import ABC
from sqlalchemy import update
from pprint import pprint
from sspo_db.model.core.models import ApplicationReference

class AbstractApplication(ABC):

    def __init__(self, service):
        self.service = service
        
    def create(self, object):
        self.service.create (object)

    def ___retrive_by_external_id_and_seon_entity_name (self, external_id, seon_entity_name):
        return self.service.session.query(ApplicationReference).filter(ApplicationReference.external_id == external_id, 
                                                               ApplicationReference.entity_name == seon_entity_name).first()

    def ___retrive_by_external_url_and_seon_entity_name (self, external_url, seon_entity_name):
        return self.service.session.query(ApplicationReference).filter(ApplicationReference.external_url == external_url, 
                                                               ApplicationReference.entity_name == seon_entity_name).first()

    def retrive_by_external_url(self, external_url):

        application_reference = self.___retrive_by_external_url_and_seon_entity_name(external_url, self.service.type)
        if application_reference:
            return self.service.get_by_uuid(application_reference.internal_uuid)
        return None

    def retrive_by_external_uuid(self, external_uuid):

        application_reference = self.___retrive_by_external_id_and_seon_entity_name(external_uuid, self.service.type)
        if application_reference:
            return self.service.get_by_uuid(application_reference.internal_uuid)
        return None
    
    def update (self, object):
        self.service.update(object)
    
    def retrive_by_name (self, name):
        return self.service.retrive_by_name(name)
    
    def get_by_uuid (self, uuid):
        return self.service.get_by_uuid(uuid)
    



from sspo_db.model.core.models import ApplicationReference
from sspo_db.service.core.service import ApplicationReferenceService, ConfigurationService, ApplicationService, ApplicationTypeService
from sspo_db.application.abstract_application import AbstractApplication


class ApplicationApplicationType(AbstractApplication):
    
    def __init__(self):
        super().__init__(ApplicationTypeService())

class ApplicationApplication(AbstractApplication):
    
    def __init__(self):
        super().__init__(ApplicationService())

class ApplicationConfiguration(AbstractApplication):
    
    def __init__(self):
        super().__init__(ConfigurationService())

class ApplicationApplicationReference(AbstractApplication):
    
    def __init__(self):
        super().__init__(ApplicationReferenceService())
        
    def get_by_external_uuid(self,external_id):
        return self.service.retrive_by_external_id(external_id)

    def get_by_external_uuid_and_seon_entity_name(self,external_id,seon_entity_name):
        return self.service.retrive_by_external_id_and_seon_entity_name(external_id, seon_entity_name)
                    
                    


from sspo_db.service.activity.service import *
from sspo_db.application.abstract_application import AbstractApplication

class ApplicationScrumIntentedDevelopmentTask(AbstractApplication):
    
    def __init__(self):
        super().__init__(ScrumIntentedDevelopmentTaskService())
    
class ApplicationScrumPerformedDevelopmentTask(AbstractApplication):
    def __init__(self):
        super().__init__(ScrumPerformedDevelopmentTaskService())
    
class ApplicationDevelopmentTaskType(AbstractApplication):

    def __init__(self):
        super().__init__(DevelopmentTaskTypeService())
    
    def retrive_by_name (self, name):
        return self.service.retrive_by_name(name)

class ApplicationPriority(AbstractApplication):

    def __init__(self):
        super().__init__(PriorityService())
    
    def retrive_by_name (self, name):
        return self.service.retrive_by_name(name)

class ApplicationRisk(AbstractApplication):

    def __init__(self):
        super().__init__(RiskService())
    
    def retrive_by_name (self, name):
        return self.service.retrive_by_name(name)

class ApplicationScrumDevelopmentTask(AbstractApplication):
    def __init__(self):
        super().__init__(ScrumDevelopmentTaskService())


from sspo_db.application.abstract_application import AbstractApplication
from sspo_db.model.process.models import Sprint, ScrumAtomicProject, ScrumComplexProject, ScrumProcess, ProductBacklogDefinition
from sspo_db.model.artifact.models import ProductBacklog
from sspo_db.model.organization.models import ScrumTeam, DevelopmentTeam
from sspo_db.application.core.application import ApplicationApplicationReference

from sspo_db.service.process.service import SprintService, ScrumComplexProjectService, ScrumAtomicProjectService, ScrumProjectService, ScrumAtomicProjectService, ScrumProcessService, ProductBacklogDefinitionService
from sspo_db.service.artifact.service import ProductBacklogService
from sspo_db.service.organization.service import ScrumTeamService, DevelopmentTeamService


class ApplicationSprint(AbstractApplication):

    def __init__(self):
        super().__init__(SprintService())
        
    def retrive_by_name_and_project_name(self, sprint_name, project_name):
        return self.service.retrive_by_name_and_project_name(sprint_name, project_name)
    
    def retrive_limbo(self, project_uuid):
        return self.service.retrive_limbo(project_uuid)
    
class ApplicationScrumComplexProject(AbstractApplication):

    def __init__(self):
        super().__init__(ScrumComplexProjectService())
    
    def get_all(self, organization_uuid):
        return self.service.get_all(organization_uuid)

    
class ApplicationScrumAtomicProject(AbstractApplication):

    def __init__(self):
        super().__init__(ScrumAtomicProjectService())
    
    def get_all(self, organization_uuid):
        return self.service.get_all(organization_uuid)
    
class ApplicationScrumProject(AbstractApplication):

    def __init__(self):
        super().__init__(ScrumProjectService())
    
    
class ApplicationScrumProcess(AbstractApplication):
    
    def __init__(self):
        super().__init__(ScrumProcessService())

class ApplicationProductBacklogDefinition(AbstractApplication):
    
    def __init__(self):
        super().__init__(ProductBacklogDefinitionService())
    